import { useSelector } from "react-redux";
import actionCreatorFactory from "typescript-fsa";
import { combineEpics } from "redux-observable";
import { map, ignoreElements, tap } from "rxjs/operators";
import { ofAction } from "@datalayer/typescript-fsa-redux-observable";
import { reducerWithInitialState } from "typescript-fsa-reducers";
export const notebookInitialState = {
    activeCell: undefined,
    kernelStatus: undefined,
    notebookChange: undefined,
    portals: [],
};
/* Selectors */
export const selectNotebook = () => useSelector((state) => {
    if (state.notebook) {
        return state.notebook;
    }
    return notebookInitialState;
});
/* Actions */
export var ActionType;
(function (ActionType) {
    ActionType["NOTEBOOK_CHANGE"] = "notebook/NOTEBOOK_CHANGE";
    ActionType["ACTIVE_CELL_CHANGE"] = "notebook/ACTIVE_CELL_CHANGE";
    ActionType["KERNEL_STATUS_CHANGE"] = "notebook/KERNEL_STATUS_CHANGE";
    ActionType["RUN"] = "notebook/RUN";
    ActionType["RUN_ALL"] = "notebook/RUN_ALL";
    ActionType["SAVE"] = "notebook/SAVE";
    ActionType["INTERRUPT"] = "notebook/INTERRUPT";
    ActionType["INSERT_ABOVE"] = "notebook/INSERT_ABOVE";
    ActionType["INSERT_BELOW"] = "notebook/INSERT_BELOW";
    ActionType["DELETE"] = "notebook/DELETE";
    ActionType["PORTAL"] = "notebook/PORTAL";
})(ActionType || (ActionType = {}));
const actionCreator = actionCreatorFactory('jupyterReact');
export const notebookActions = {
    notebookChange: actionCreator.async(ActionType.NOTEBOOK_CHANGE),
    activeCellChange: actionCreator.async(ActionType.ACTIVE_CELL_CHANGE),
    kernelStatusChanged: actionCreator.async(ActionType.KERNEL_STATUS_CHANGE),
    run: actionCreator.async(ActionType.RUN),
    save: actionCreator.async(ActionType.SAVE),
    runAll: actionCreator.async(ActionType.RUN_ALL),
    interrupt: actionCreator.async(ActionType.INTERRUPT),
    insertAbove: actionCreator.async(ActionType.INSERT_ABOVE),
    insertBelow: actionCreator.async(ActionType.INSERT_BELOW),
    delete: actionCreator.async(ActionType.DELETE),
    portal: actionCreator.async(ActionType.PORTAL),
};
/* Epics */
export const notebookEpics = (notebookLumino) => {
    const activeCellChangeEpic = action$ => action$.pipe(ofAction(notebookActions.activeCellChange.started), map(action => {
        return notebookActions.activeCellChange.done({
            params: action.payload,
            result: action.payload
        });
    }));
    const notebookChangeEpic = action$ => action$.pipe(ofAction(notebookActions.notebookChange.started), map(action => {
        return notebookActions.notebookChange.done({
            params: action.payload,
            result: action.payload
        });
    }));
    const kernelStatusChangeEpic = action$ => action$.pipe(ofAction(notebookActions.kernelStatusChanged.started), map(action => {
        return notebookActions.kernelStatusChanged.done({
            params: action.payload,
            result: action.payload
        });
    }));
    const runEpic = action$ => action$.pipe(ofAction(notebookActions.run.started), tap(action => {
        notebookLumino.commands.execute('notebook:run-cell');
    }), ignoreElements());
    const runAllEpic = action$ => action$.pipe(ofAction(notebookActions.runAll.started), tap(action => {
        notebookLumino.commands.execute('notebook:run-all-cells');
    }), ignoreElements());
    const saveEpic = action$ => action$.pipe(ofAction(notebookActions.save.started), tap(action => {
        notebookLumino.commands.execute('notebook:save');
    }), ignoreElements());
    const interruptEpic = action$ => action$.pipe(ofAction(notebookActions.interrupt.started), tap(action => {
        notebookLumino.commands.execute('notebook:interrupt-kernel');
    }), ignoreElements());
    const insertAboveEpic = action$ => action$.pipe(ofAction(notebookActions.insertAbove.started), tap(action => {
        notebookLumino.commands.execute('notebook-cells:insert-above');
    }), ignoreElements());
    const insertBelowEpic = action$ => action$.pipe(ofAction(notebookActions.insertBelow.started), tap(action => {
        notebookLumino.commands.execute('notebook-cells:insert-below');
    }), ignoreElements());
    const deleteEpic = action$ => action$.pipe(ofAction(notebookActions.delete.started), tap(action => {
        notebookLumino.commands.execute('notebook-cells:delete');
    }), ignoreElements());
    const portalEpic = action$ => action$.pipe(ofAction(notebookActions.portal.started), map(action => {
        return notebookActions.portal.done({
            params: action.payload,
            result: action.payload
        });
    }));
    const loggingEpic = action$ => action$
        .pipe(ofAction(notebookActions.notebookChange.started, notebookActions.kernelStatusChanged.started, notebookActions.run.started, notebookActions.runAll.started, notebookActions.interrupt.started, notebookActions.delete.started, notebookActions.insertAbove.started, notebookActions.insertBelow.started, notebookActions.portal.started), tap(action => console.log(action.type)), ignoreElements());
    return combineEpics(portalEpic, activeCellChangeEpic, kernelStatusChangeEpic, notebookChangeEpic, runEpic, runAllEpic, saveEpic, interruptEpic, insertAboveEpic, insertBelowEpic, deleteEpic, loggingEpic);
};
/* Reducers */
export const notebookReducer = reducerWithInitialState(notebookInitialState)
    .case(notebookActions.activeCellChange.done, (state, success) => {
    return Object.assign(Object.assign({}, state), { activeCell: success.result });
})
    .case(notebookActions.notebookChange.done, (state, success) => {
    return Object.assign(Object.assign({}, state), { notebookChange: success.result });
})
    .case(notebookActions.kernelStatusChanged.done, (state, success) => {
    return Object.assign(Object.assign({}, state), { kernelStatus: success.result });
})
    .case(notebookActions.portal.done, (state, success) => {
    return Object.assign(Object.assign({}, state), { portals: state.portals.concat([success.result]) });
});
//# sourceMappingURL=NotebookState.js.map